@@ -18,13 +18,14 @@ Itcan be used as a handy facility for running the task from a command line.
 .. currentmodule:: dharma_cli.cli
 .. moduleauthor:: Kodexa, Inc <support@kodexa.com>
 """
+import json
 import logging
 import os
-import sys
+import os.path
+import tarfile
 
 import click
 from rich import print
-from rich.table import Table
 
 from kodexa.cloud.kodexa import ExtensionHelper, KodexaPlatform
 
@@ -36,76 +37,6 @@ LOGGING_LEVELS = {
     4: logging.DEBUG,
 }  #: a mapping of `verbose` option counts to logging levels
 
-DEFAULT_COLUMNS = {
-    'extensionPacks': [
-        'orgSlug',
-        'slug',
-        'version',
-        'name',
-        'description',
-        'type',
-        'status'
-    ],
-    'default': [
-        'orgSlug',
-        'slug',
-        'version',
-        'name',
-        'description',
-        'type'
-    ]
-}
-
-OBJECT_TYPES = {
-    "extensionPacks": {
-        "name": "extension pack",
-        "plural": "extension packs"
-    },
-    "pipelines": {
-        "name": "pipelines",
-        "plural": "pipelines"
-    },
-    "actions": {
-        "name": "action",
-        "plural": "actions"
-    },
-    "stores": {
-        "name": "store",
-        "plural": "stores"
-    },
-    "connectors": {
-        "name": "connector",
-        "plural": "connectors"
-    },
-    "taxonomies": {
-        "name": "taxonomy",
-        "plural": "taxonomies"
-    },
-    "workflows": {
-        "name": "workflow",
-        "plural": "workflows"
-    }
-}
-
-
-def resolve_object_type(obj_type):
-    hits = []
-    keys = []
-    for target_type in OBJECT_TYPES.keys():
-        if obj_type in target_type:
-            hits.append(OBJECT_TYPES[target_type])
-            keys.append(target_type)
-
-    if len(hits) == 1:
-        return keys[0], hits[0]
-
-    if len(hits) == 0:
-        print(":exclaimation: Unable to find object type {obj_type}")
-        sys.exit(1)
-    else:
-        print(f":exclaimation: To many potential matches for object type ({','.join(keys)}")
-        sys.exit(1)
-
 
 class Info(object):
     """An information object to pass data between CLI functions."""
@@ -171,58 +102,29 @@ def deploy(_: Info, path: str, url: str, org: str, token: str):
 
 @cli.command()
 @click.argument('object_type')
-@click.argument('organization_slug')
+@click.argument('ref')
 @click.option('--url', default=KodexaPlatform.get_url(), help='The URL to the Kodexa server')
 @click.option('--token', default=KodexaPlatform.get_access_token(), help='Access token')
 @pass_info
-def get(_: Info, object_type: str, organization_slug: str, url: str, token: str):
+def get(_: Info, object_type: str, ref: str, url: str, token: str):
     """List the instance of the object type"""
     KodexaPlatform.set_url(url)
     KodexaPlatform.set_access_token(token)
-    object_type, object_type_metadata = resolve_object_type(object_type)
-    try:
-        objects = KodexaPlatform.list_objects(organization_slug, object_type)
-
-        cols = DEFAULT_COLUMNS['default']
-
-        if object_type in DEFAULT_COLUMNS:
-            cols = DEFAULT_COLUMNS[object_type]
-
-        print("\n")
-        table = Table(title=f"Listing {object_type_metadata['plural']}")
-        for col in cols:
-            table.add_column(col)
-        for object_dict in objects['content']:
-            row = []
-
-            for col in cols:
-                row.append(object_dict[col] if col in object_dict else '')
-            table.add_row(*row)
-
-        print(table)
-    except:
-        print(f"\n:exclamation: Failed to get {object_type_metadata['name']} [{sys.exc_info()[0]}]")
+    KodexaPlatform.get(object_type, ref)
 
 
 @cli.command()
 @click.argument('object_type')
-@click.argument('organization_slug')
-@click.argument('slug')
+@click.argument('ref')
 @click.option('--url', default=KodexaPlatform.get_url(), help='The URL to the Kodexa server')
 @click.option('--token', default=KodexaPlatform.get_access_token(), help='Access token')
 @pass_info
-def delete(_: Info, object_type: str, organization_slug: str, slug: str, url: str, token: str):
+def delete(_: Info, object_type: str, ref: str, url: str, token: str):
     """Delete object from the platform"""
-    object_type, object_type_metadata = resolve_object_type(object_type)
 
-    print(f"Deleting {object_type_metadata['name']} [bold]{organization_slug}/{slug}[/bold]")
     KodexaPlatform.set_url(url)
     KodexaPlatform.set_access_token(token)
-    try:
-        KodexaPlatform.delete_object(organization_slug, slug, object_type)
-        print(f"Deleted {object_type_metadata['name']} [bold]{organization_slug}/{slug}[/bold] :tada:")
-    except:
-        print(f"\n:exclamation: Failed to delete {object_type_metadata['name']} [{sys.exc_info()[0]}]")
+    KodexaPlatform.delete(object_type, ref)
 
 
 @cli.command()
@@ -243,4 +145,43 @@ def document(_: Info, path: str):
     print("Metadata loaded")
     from kodexa.cli.documentation import generate_documentation
     generate_documentation(metadata)
-    print("Documentation has been successfully built :tada:")
+    print("Extension documentation has been successfully built :tada:")
+
+
+@cli.command()
+@click.option('--path', default=os.getcwd(), help='Path to folder container kodexa.yml (defaults to current)')
+@click.option('--output', default=os.getcwd() + "/dist",
+              help='Path to the output folder (defaults to dist under current)')
+@click.option('--version', default=os.getenv('VERSION'), help='Version number (defaults to 1.0.0)')
+@pass_info
+def package(_: Info, path: str, output: str, version: str):
+    """Load metadata"""
+    metadata_obj = ExtensionHelper.load_metadata(path)
+    print("Preparing to pack")
+    try:
+        os.makedirs(output)
+    except OSError as e:
+        import errno
+        if e.errno != errno.EEXIST:
+            raise
+
+    metadata_obj['version'] = version if version is not None else '1.0.0'
+
+    if 'source' in metadata_obj and 'location' in metadata_obj['source']:
+        metadata_obj['source']['location'] = metadata_obj['source']['location'].format(**metadata_obj)
+
+    versioned_metadata = os.path.join(output, f"{metadata_obj['slug']}-{metadata_obj['version']}.json")
+    unversioned_metadata = os.path.join(output, "kodexa.json")
+    with open(versioned_metadata, 'w') as outfile:
+        json.dump(metadata_obj, outfile)
+
+    from shutil import copyfile
+    copyfile(versioned_metadata, unversioned_metadata)
+
+    output_filename = f"{metadata_obj['slug']}-{metadata_obj['version']}.tar.gz"
+    with tarfile.open(output_filename, "w:gz") as tar:
+        tar.add(output, arcname=os.path.basename(output))
+
+    os.rename(output_filename, os.path.join(output, output_filename))
+
+    print("Extension has been packaged :tada:")
